/*
 * -------------------------------------------
 *    MSP432 DriverLib - v3_21_00_05 
 * -------------------------------------------
 *
 * --COPYRIGHT--,BSD,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 * MSP432 DMA - eUSCI SPI Transfer Using DMA
 *
 * Description: In this code example, the MSP432 's DMA controller is used in
 * conjunction with an SPI loopback configuration to demonstrate how to use
 * hardware triggered DMA transfers. Four DMA transfers are setup using four
 * separate DMA channels. For each SPI instance, a DMA channel is setup to
 * receive and transfer from the SIMO and SOMI ports respectively. After an
 * arbitrary string is sent through SPI via the loopback configuration, an
 * ISR counter is incremented and triggers a NOP condition in the main loop
 * where the user can set a breakpoint to examine memory contents.
 *
 *
 *                      MSP432P401
 *             ---------------------------
 *         /|\|                          |
 *          | |                          |
 *          --|RST           P1.5 (CLK)  |---------
 *            |              P1.7 (SIMO) |----------
 *            |              P1.6 (SOMI) |--------
 *            |                          |
 *            |                          |
 *
 * Author: Timothy Logan
 ******************************************************************************/

/* DriverLib Includes */
#include "driverlib.h"

/* Standard Includes */
#include <stdint.h>
#include <string.h>
#include <stdbool.h>

#define INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE	18

/* SPI Configuration Parameter */
const eUSCI_SPI_MasterConfig spiMasterConfig =
{
EUSCI_B_SPI_CLOCKSOURCE_SMCLK, // SMCLK Clock Source
6000000, // SMCLK = DCO/2 = 12MHz/2 = 6MHz
6000000, // SPICLK = 6MHz
EUSCI_B_SPI_MSB_FIRST, // MSB First
EUSCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT, // Phase
EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_HIGH, // High polarity
EUSCI_B_SPI_3PIN // 3Wire SPI Mode
};

/* DMA Control Table */
#if defined(__TI_COMPILER_VERSION__)
#pragma DATA_ALIGN(MSP_EXP432P401RLP_DMAControlTable, 1024)
#elif defined(__IAR_SYSTEMS_ICC__)
#pragma data_alignment=1024
#elif defined(__GNUC__)
__attribute__ ((aligned (1024)))
#elif defined(__CC_ARM)
__align(1024)
#endif
static DMA_ControlTable MSP_EXP432P401RLP_DMAControlTable[16];

uint8_t mRXData[INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE];
uint8_t mTXData[INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE];


int main(void)
{
    volatile uint32_t ii;
    
    /* Halting Watchdog */
    MAP_WDT_A_holdTimer();
    MAP_Interrupt_enableSleepOnIsrExit();

    for(ii=0;ii<INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE;ii++)
    {
    	mTXData[ii] = ii;
    }

    /*
     * Revision C silicon supports wait states of 0 at 12Mhz
     */
    MAP_PCM_setCoreVoltageLevel(PCM_VCORE0);
    MAP_FlashCtl_setWaitState(FLASH_BANK0, 0);
    MAP_FlashCtl_setWaitState(FLASH_BANK1, 0);

    /*
     * Setting up clocks
     * MCLK = DCO = 12MHz
     * SMCLK = DCO/2 = 6Mhz
     * ACLK = REFO = 32Khz
     */
    MAP_CS_setDCOFrequency(12000000);
    MAP_CS_initClockSignal(CS_ACLK, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    MAP_CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_2);
    MAP_CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);

    /* Configure CLK, MOSI & MISO for SPI0 (EUSCI_B0) */
    MAP_GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1,
            GPIO_PIN5 | GPIO_PIN6, GPIO_PRIMARY_MODULE_FUNCTION);
    MAP_GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1,
            GPIO_PIN7, GPIO_PRIMARY_MODULE_FUNCTION);

    /*
     * Configuring P1.1 as an input and enabling interrupt, the timer is started from
     * GPIO ISR.
     */
    MAP_GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P1, GPIO_PIN1);
    MAP_GPIO_interruptEdgeSelect(GPIO_PORT_P1,GPIO_PIN1,GPIO_HIGH_TO_LOW_TRANSITION);
    MAP_GPIO_clearInterruptFlag(GPIO_PORT_P1, GPIO_PIN1);
    MAP_GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN1);
    MAP_Interrupt_enableInterrupt(INT_PORT1);

    MAP_GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN4);
    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN4);

    /* Configuring SPI module */
    MAP_SPI_initMaster(EUSCI_B0_BASE, &spiMasterConfig);

    /* Enable the SPI module */
    MAP_SPI_enableModule(EUSCI_B0_BASE);

    /* Configuring DMA module */
    MAP_DMA_enableModule();
    MAP_DMA_setControlBase(MSP_EXP432P401RLP_DMAControlTable);

    /* Assign DMA channel 0 to EUSCI_B0_TX0, channel 1 to EUSCI_B0_RX0 */
    MAP_DMA_assignChannel(DMA_CH0_EUSCIB0TX0);
    MAP_DMA_assignChannel(DMA_CH1_EUSCIB0RX0);

    /* Setup the TX transfer characteristics & buffers */
    MAP_DMA_setChannelControl(DMA_CH0_EUSCIB0TX0 | UDMA_PRI_SELECT,
    UDMA_SIZE_8 | UDMA_SRC_INC_8 | UDMA_DST_INC_NONE | UDMA_ARB_1);

    MAP_DMA_setChannelTransfer(DMA_CH0_EUSCIB0TX0 | UDMA_PRI_SELECT, UDMA_MODE_BASIC,
    mTXData, (void *) MAP_SPI_getTransmitBufferAddressForDMA(EUSCI_B0_BASE),
    INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE);

    /* Setup the RX transfer characteristics & buffers */
    MAP_DMA_setChannelControl(DMA_CH1_EUSCIB0RX0 | UDMA_PRI_SELECT,
    UDMA_SIZE_8 | UDMA_SRC_INC_NONE | UDMA_DST_INC_8 | UDMA_ARB_1);

    MAP_DMA_setChannelTransfer(DMA_CH1_EUSCIB0RX0 | UDMA_PRI_SELECT, UDMA_MODE_BASIC,
    (void *) MAP_SPI_getReceiveBufferAddressForDMA(EUSCI_B0_BASE), mRXData,
    INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE);

    // Enable DMA interrupt and clear any previously set completion flags

    MAP_DMA_enableInterrupt(INT_DMA_INT0);
    MAP_Interrupt_enableInterrupt(INT_DMA_INT0);
    MAP_DMA_clearInterruptFlag(DMA_CH0_EUSCIB0TX0);
    MAP_DMA_clearInterruptFlag(DMA_CH1_EUSCIB0RX0);

    /* Polling to see if the master receive is finished */
    while(1);
}

void DMA_INT0_IRQHandler(void)
{
	uint32_t int0_srcflg_dma = DMA_Channel->INT0_SRCFLG;

//	if (int0_srcflg_dma & DMA_INT0_SRCFLG_CH0) // Transmit DMA completed
//	{
//	MAP_DMA_clearInterruptFlag(DMA_CH0_EUSCIB0TX0); // clear the TX DMA interrupt flag
//	MAP_DMA_disableChannel(DMA_CHANNEL_0); // disable the TX channel
//	}
//	else if (int0_srcflg_dma & DMA_INT0_SRCFLG_CH1) // Recieve DMA completed
//	{
//	MAP_DMA_clearInterruptFlag(DMA_CH1_EUSCIB0RX0); // clear the RX DMA interrupt flag
//	MAP_DMA_disableChannel(DMA_CHANNEL_1); // disable the RX channel
//	}
//	else
//	{
//	__nop();
//	return;
//	}

	if (int0_srcflg_dma & DMA_INT0_SRCFLG_CH1) // Recieve DMA completed
	{
	    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN4);
		MAP_DMA_clearInterruptFlag(DMA_CH0_EUSCIB0TX0); // clear the TX DMA interrupt flag
		MAP_DMA_disableChannel(DMA_CHANNEL_0); // disable the TX channel
        MAP_DMA_clearInterruptFlag(DMA_CH1_EUSCIB0RX0); // clear the RX DMA interrupt flag
        MAP_DMA_disableChannel(DMA_CHANNEL_1); // disable the RX channel

	    //deassert the chip select line to indicate end of SPI transmission
	    /*
	     * Reset DMA channels 0 and 1, size and mode see SLAU356 page 466, at
	     * the end of cycle the count is 0 and the cycle_ctrl is automatically.
	     * Alternatively you can re-use the setChannelControl and Transfer APIs.
	     */

		MSP_EXP432P401RLP_DMAControlTable[0].control =
				(MSP_EXP432P401RLP_DMAControlTable[0].control & 0xff000000 ) |
				(((INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE)-1)<<4) | 0x01;
		MSP_EXP432P401RLP_DMAControlTable[1].control =
				(MSP_EXP432P401RLP_DMAControlTable[1].control & 0xff000000 ) |
				(((INERTIAL_NUM_BYTES_TO_READ_PER_SAMPLE)-1)<<4) | 0x01;

	    // set the DMA transfer complete flag
	    //mDMAXferComplete = true;
	}
}

void PORT1_IRQHandler(void)
{
	if(P1->IV == 4)
	{
		P1OUT |= BIT0;
		while(P1->IFG & BIT1)
		{
			P1->IFG &= ~BIT1;
		}
		/* first assert the Chip Select (CS) to signal the start of the SPI transmission */
		MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN4);

	    MAP_DMA_enableChannel(DMA_CHANNEL_0);
	    MAP_DMA_enableChannel(DMA_CHANNEL_1);
		P1OUT &= ~BIT0;
	}
}
